/**********************************************************************************************/
/* VDK_ObjectPtr.h																			  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

/*
	This class is used to establish relation 1 : M beetwen two Tables by 'direct pointer'
	
	Let exists Table_A and Tables_B.
	Let Tables_B have field ObjectPtr which point on Table_A.
	Then we have relation [Table_A : Tables_B] as [1 : M]
	
	IF the Table_A is owner of relation, and IF we delete some record 
	in Table_A (parent record), then will be deleted all its child records in Tables_B.
	ELSE pointer in childs records will be set to *NULL* value.
	
*/

#ifndef __VDK_ObjectPtr_h 
	#define __VDK_ObjectPtr_h
	
#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/VDK_ULong.h>
#	include <VSDK/VDK_Link.h>
#else
#	include <VDK_ULong.h>
#	include <VDK_Link.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class VDK_BaseObject;
 

/**********************************************************************************************/
class FBL_EXP VDK_ObjectPtr : public VDK_ULong, public VDK_Link
{
		friend class VDK_BaseObject; 

	public://///////////////////////////////////////////////////////////////////////////////////
		
FBL_EXP							VDK_ObjectPtr( 	
									const char*			inName,
									VDK_BaseObject*		inPointedObject	  = NULL,
									FBL_DeletionControl	inDeletionControl = kFBL_SET_NULL );

FBL_EXP virtual 				~VDK_ObjectPtr( void );	
								
								
		VDK_BaseObject*			GetPointedObject( void ) const		{ return mPointedObject; }
		FBL_DeletionControl		GetDeletionControl( void ) const	{ return mDeletionControl;}
		
FBL_EXP	void					SetPointedObject( 
									VDK_BaseObject* 	inPointedObject,
									bool				inSaveOnDisk = true );
									
FBL_EXP	void					SetDeletionControl( 
									FBL_DeletionControl inDeletionControl,
									bool				inSaveOnDisk = true );

		/* Value methods */
		
FBL_EXP	void					operator =( FBL_REC_ID inValue ); 
FBL_EXP							operator FBL_REC_ID( void ) const; 

		/* Search methods */
		
								// For ObjectPtr there is no sense for range search, so
								// there is only exact search by one key:
		FBL_BitSet* 			Find( FBL_REC_ID* inKey = NULL ) {
									return VDK_Field::Find( inKey );
								}
		
		
		/* RDB - conversion: */
	
FBL_EXP	virtual	void			ConvertFromRDB( VDK_Field* inRDB_KEY,
												VDK_Field* inRDB_PTR  );
	
		/* LINK ACCESSORIES: */	
	
virtual	FBL_Link_Type			GetLink_Type( void ) const { return kFBL_ObjectPointer; 	}
virtual	const char*				GetLinkName( void )	const { return (char*) GetName();	}	
	
	
		/* BRANCHES METHODES: */

FBL_EXP	virtual	bool			AllowDeletionFor( VDK_BaseObject* inObject );

								// Returns number of branches in relation.	
virtual ushort					GetBranchesCount( void ) const	{ return 2; }
	
								// Returns BaseObject for specified branch.
virtual VDK_BaseObject*			GetObject( ushort BranchNumber ) const;

								// Returns parameters for specified branch.
virtual void					GetBranchParameters( 
										ushort 					BranchNumber, 
										FBL_RelType& 			outRelType,
										FBL_DeletionControl& 	outDeletionControl ) const;

								// Cnhange paramters of existing branch.
FBL_EXP virtual void			ChangeBranchParameters( 
										ushort 					inBranchNumber,
										FBL_RelType				inNewRelType,
										FBL_DeletionControl 	inDeletionControl );

								// Returns the name of specified link in direction from
								// inBaseObjectA to inBaseObjectB.
FBL_EXP virtual	const char*		GetNameFor( 
									VDK_BaseObject* inBaseObjectA,
									VDK_BaseObject* inBaseObjectB )	const;


		/* LINK RECORDS METHODES: */
										
								// Add to relation new record with given set of keys.
								// Array must contains right count of keys, in order of branches.
FBL_EXP virtual bool			AddLinkRecord( FBL_REC_ID Keys[] );		

								// Remove from relation record which contains given set of keys.
FBL_EXP virtual void			RemoveLinkRecord( FBL_REC_ID Keys[] );	
		

	protected://////////////////////////////////////////////////////////////////////////////////

FBL_EXP virtual void			FinishCreateSelf( void );
		
								
	protected://////////////////////////////////////////////////////////////////////////////////

		VDK_BaseObject*			mPointedObject;		// BaseObject pointed by this pointer.
		FBL_DeletionControl		mDeletionControl;	
}; 


/**********************************************************************************************/
inline VDK_BaseObject* VDK_ObjectPtr::GetObject( ushort BranchNumber ) const
{ 
	// The first branch is my own object (Many), 
	// the second branch is pointed object (One)	

	CHECK_FBL( BranchNumber <= 2 ); // 1 or 2
	return (BranchNumber == 1) ? mBaseObject : mPointedObject;
}


/**********************************************************************************************/
inline void VDK_ObjectPtr::GetBranchParameters( 
	ushort 					BranchNumber, 
	FBL_RelType&			outRelType,
	FBL_DeletionControl& 	outDeletionControl ) const
{ 
	// The first branch is my own object (Many), 
	// the second branch is pointed object (One)	

	CHECK_FBL( BranchNumber == 1 || BranchNumber == 2 ); 
	
	// F_ObjectPointer is relation beetwen object as [M : 1] or [1 : 1]
	outRelType = (BranchNumber == 1) ? (CheckFlags( fUnique ) ? FBL_One : FBL_Many) : 
										FBL_One;
										
	outDeletionControl = (BranchNumber == 1) ? 
							kFBL_SET_NULL :			// we always can delete Many record.
							mDeletionControl;
}


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif  


